unit thread_splash_unit;

interface

uses
  Classes, windows, SysUtils,thread_splash_form,forms;

type
  TSplashThread = class(TThread)
  private
    FFormThreadSplash:TFormThreadSplash;

    redraw,closeSplash:boolean;
    text_tmp:string;
    pos_tmp:integer;

    procedure Draw_splash_sync;
    procedure CloseSplash_sync;
    procedure Close_sync;
    procedure Show_sync;
    procedure SetText_sync;
    procedure SetProgress_sync;
  public
    Constructor Create(AOwner: TComponent);
    destructor Destroy; override;

    procedure Execute; override;
    procedure Stop;
    procedure Start;

    procedure SetProgress(value:integer);
    procedure SetText(text:string);
  end;

implementation

Constructor TSplashThread.Create(AOwner: TComponent);
begin
  inherited create(true);
  FreeOnTerminate:=true;
  Priority:=tpNormal;
  redraw:=false;
  closeSplash:=false;
  text_tmp:='';
  pos_tmp:=0;
  FFormThreadSplash:=TFormThreadSplash.Create(AOwner);
end;

destructor TSplashThread.Destroy;
begin
 Terminate;
 inherited;
end;

procedure Delay(dwMilliseconds: Longint);
var iStart,iStop:DWORD;
begin
  iStart := GetTickCount;
  repeat
    iStop := GetTickCount;
    sleep(1);
  until (iStop - iStart) >= dwMilliseconds;
end;

procedure TSplashThread.Draw_splash_sync;
begin
 FFormThreadSplash.Draw_splash;
end;

procedure TSplashThread.CloseSplash_sync;
begin
 FFormThreadSplash.CloseSplash;
end;

procedure TSplashThread.Close_sync;
begin
 FFormThreadSplash.Close;
end;

procedure TSplashThread.Show_sync;
begin
 FFormThreadSplash.Show
end;

procedure TSplashThread.Execute;
begin
 while not(terminated) do begin
   if(redraw) then begin
    Synchronize(Draw_splash_sync);
    redraw:=false;
   end;
   if(closeSplash) then begin
     Delay(1000);
     // ,     :)
     Synchronize(CloseSplash_sync);
     Delay(3000);
     Synchronize(Close_sync);
     terminate;
   end;
  sleep(1);
 end;
end;


procedure TSplashThread.Stop;
begin
 closeSplash:=true;
end;

procedure TSplashThread.Start;
begin
 Synchronize(Show_sync);
 resume;
end;

//, ...
procedure TSplashThread.SetText_sync;
begin
 FFormThreadSplash.SetText(text_tmp);
end;

procedure TSplashThread.SetProgress_sync;
begin
 FFormThreadSplash.SetProgress(pos_tmp);
end;

procedure TSplashThread.SetText(text:string);
begin
 text_tmp:=text;
 Synchronize(SetText_sync);
 redraw:=true;
end;

procedure TSplashThread.SetProgress(value:integer);
begin
 pos_tmp:=value;
 Synchronize(SetProgress_sync);
 redraw:=true;
end;


end.
